# enmity.py

from __future__ import division

import random

import math
from OpenGL.GL import *

import humanity



class Ninja(humanity.Character):
    GRAVITY = 70.0

    TEXTURE_NAMES = (
        'ninja-leftstep.png',
        'ninja-leftmidstep.png',
        'ninja-rightstep.png',
        'ninja-rightmidstep.png',
        )

    RUNFRAMES = ('leftmidstep','rightstep','rightmidstep','leftstep')

    def __init__(self):
        super(Ninja,self).__init__()

    def enter_scene(self,scene,x,y):
        super(Ninja,self).enter_scene(scene,x,y)
        self.life = 10.0
        self.power = 0.1

    def exit_scene(self):
        del self.life
        del self.power
        super(Ninja,self).exit_scene()

    def draw(self):
        if self.facing == 'right':
            x0 = self.x - 2.2
            x1 = self.x + 2.2
        else:
            x0 = self.x + 2.2
            x1 = self.x - 2.2
        if self.inair:
            frame = "leftmidstep"
        else:
            frame = self.RUNFRAMES[self.runframe]
        texture = self.textures["ninja-%s.png" % frame]
        texture.bind()
        glBegin(GL_QUADS)
        glColor(1.0,1.0,1.0,1.0)
        glTexCoord(0.0,0.0)
        glVertex(x0,self.y)
        glTexCoord(0.0,1.0)
        glVertex(x0,self.y+6.5)
        glTexCoord(1.0,1.0)
        glVertex(x1,self.y+6.5)
        glTexCoord(1.0,0.0)
        glVertex(x1,self.y)
        glEnd()
        texture.unbind()


class FreeNinja(Ninja):
    SPEED = 10.0

    def enter_scene(self,scene,x,y):
        super(FreeNinja,self).enter_scene(scene,x,y)
        if self.x < self.scene.heroine.x:
            self.facing = "right"
            self.xvelcommand = self.SPEED
        else:
            self.facing = "left"
            self.xvelcommand = -self.SPEED

    def poll_interactions(self):
        pass


class JumpingNinja(FreeNinja):
    SPEED = 15.0
    JUMP = 170.0

    def enter_scene(self,scene,x,y):
        super(JumpingNinja,self).enter_scene(scene,x,y)
        self.jumptimer = None

    def exit_scene(self):
        del self.jumptimer
        super(Ninja,self).exit_scene()

    def poll_interactions(self):
        dist = abs(self.x - self.scene.heroine.x)
        if self.jumptimer is not None:
            self.jumpcommand = True
        elif not self.inair and 8.0 < dist < 12.0:
            self.jumpcommand = True
            self.jumptimer = 0.295
        else:
            self.jumpcommand = False
        if self.x > self.scene.heroine.x:
            self.xvelcommand = -self.SPEED
            self.facing = "left"
        else:
            self.xvelcommand = self.SPEED
            self.facing = "right"

    def update(self,dt):
        if self.jumptimer is not None:
            self.jumptimer -= dt
            if self.jumptimer <= 0.0:
                self.jumptimer = None
        super(JumpingNinja,self).update(dt)


class BoundNinja(Ninja):
    SPEED = 5.0

    def enter_scene(self,scene,x,y,xmin,xmax):
        super(BoundNinja,self).enter_scene(scene,x,y)
        self.xmax = xmax
        self.xmin = xmin

    def exit_scene(self):
        del self.xmax
        del self.xmin
        super(BoundNinja,self).exit_scene()

    def poll_interactions(self):
        if self.facing == "right":
            if self.x < self.xmax:
                self.xvelcommand = self.SPEED
            else:
                self.xvelcommand = -self.SPEED
                self.facing = "left"
        else:
            if self.x > self.xmin:
                self.xvelcommand = -self.SPEED
            else:
                self.xvelcommand = self.SPEED
                self.facing = "right"
