# heroism.py

from __future__ import division

import math

import pygame
from OpenGL.GL import *
from OpenGL.GLU import *

import humanity, controls


class Heroine(humanity.Character):
    HEIGHT = 6.0
    SPEED = 20.0
    JUMP = 170.0
    GRAVITY = 70.0
    STEAMRATE = 0.08
    STEAMFRAMETIME = 0.045
    RELIEFRATE = 0.2

    TEXTURE_NAMES = (
        'right-heroine-standing.png',
        'right-heroine-leftstep.png',
        'right-heroine-midstep.png',
        'right-heroine-rightstep.png',
        'right-heroine-thrust.png',
        'right-heroine-slash.png',
        'right-heroine-hack1.png',
        'right-heroine-hack2.png',
        'right-heroine-hack3.png',
        'right-heroine-steaming.png',
        #'right-heroine-rolling.png',
        'left-heroine-standing.png',
        'left-heroine-leftstep.png',
        'left-heroine-midstep.png',
        'left-heroine-rightstep.png',
        'left-heroine-thrust.png',
        'left-heroine-slash.png',
        'left-heroine-hack1.png',
        'left-heroine-hack2.png',
        'left-heroine-hack3.png',
        'left-heroine-steaming.png',
        #'left-heroine-rolling.png',
        'life.png',
        'steam.png',
        'steam1.png',
        'steam2.png',
        'steam3.png',
        'steam4.png',
        'numbers.png',
        )

    RUNFRAMES = ('midstep','rightstep','midstep','leftstep')

    def __init__(self):
        super(Heroine,self).__init__()
        self.kills = 0

    def enter_scene(self,scene,x,y):
        super(Heroine,self).enter_scene(scene,x,y)
        self.leftdown = False
        self.rightdown = False
        self.updown = False
        self.downdown = False
        self.firedown = False
        self.jumpdown = False
        self.firepress = 0
        self.jumppress = 0
        self.attack = None
        self.attacktimer = None
        self.jumptimer = None
        self.steamtimer = None
        self.steamframe = 0
        self.steamframetimer = self.STEAMFRAMETIME
        self.steam = 0.0
        self.life = 1.0
        self.tension = 1
        self.kills = 0

    def exit_scene(self):
        del self.leftdown
        del self.rightdown
        del self.updown
        del self.downdown
        del self.firedown
        del self.jumpdown
        del self.firepress
        del self.jumppress
        del self.attack
        del self.attacktimer
        del self.jumptimer
        del self.steamtimer
        del self.steamframe
        del self.steamframetimer
        del self.steam
        del self.life
        super(Heroine,self).exit_scene()

    def process_input(self):
        for event in pygame.event.get():
            if event.type == pygame.KEYDOWN:
                f = controls.KEYMAP.get(event.key)
                if f == "left":
                    self.leftdown = True
                elif f == "right":
                    self.rightdown = True
                elif f == "up":
                    self.updown = True
                elif f == "down":
                    self.downdown = True
                elif f == "attack":
                    self.firedown = True
                    self.firepress += 1
                elif f == "jump":
                    self.jumpdown = True
                    self.jumppress += 1
                elif f == "quit":
                    raise SystemExit
            elif event.type == pygame.KEYUP:
                f = controls.KEYMAP.get(event.key)
                if f == "left":
                    self.leftdown = False
                elif f == "right":
                    self.rightdown = False
                elif f == "up":
                    self.updown = False
                elif f == "down":
                    self.downdown = False
                elif f == "attack":
                    self.firedown = False
                elif f == "jump":
                    self.jumpdown = False
            elif event.type == pygame.JOYBUTTONDOWN:
                f = controls.JOYBUTTONMAP.get(event.button)
                if f == "attack":
                    self.firedown = True
                    self.firepress += 1
                elif f == "jump":
                    self.jumpdown = True
                    self.jumppress += 1
                elif f == "quit":
                    raise SystemExit
            elif event.type == pygame.JOYBUTTONUP:
                f = controls.JOYBUTTONMAP.get(event.button)
                if f == "attack":
                    self.firedown = False
                elif f == "jump":
                    self.jumpdown = False
            elif event.type == pygame.JOYAXISMOTION:
                f = controls.JOYAXISMAP.get(event.axis)
                if f in ("left","right"):
                    self.rightdown = event.value > 0.5
                    self.leftdown = event.value < -0.5
                elif f in ("up","down"):
                    self.updown = event.value < -0.5
                    self.downdown = event.value > 0.5
            elif event.type == pygame.QUIT:
                raise SystemExit

    def hurt(self,enemy):
        power = enemy.power
        if self.steamtimer is not None:
            power *= 3.0
        else:
            power *= 1 + 0.25*self.steam
        self.life -= power

    def hwedge(self,pct):
        if self.facing == "right":
            return -math.pi*pct,math.pi*pct
        else:
            return math.pi*(1-pct),-math.pi*(1-pct)

    def upwedge(self,pct):
        return math.pi*(0.5-pct),math.pi*(0.5+pct)

    def downwedge(self,pct):
        return -math.pi*(0.5+pct),-math.pi*(0.5-pct)

    def poll_interactions(self):
        if self.life <= 0.0:
            return
        apples,espressos = self.scene.grab_object(self.x,self.y)
        if apples:
            self.life = min(self.life + 0.2*apples,1.0)
        if espressos:
            self.tension += espressos
        self.xvelcommand = self.SPEED * (self.rightdown - self.leftdown)
        if self.xvelcommand > 0:
            self.facing = "right"
            if self.frametimer is None:
                self.frametimer = 0.0
        elif self.xvelcommand < 0:
            self.facing = "left"
            if self.frametimer is None:
                self.frametimer = 0.0
        else:
            self.frametimer = None
        if self.jumptimer is not None:
            self.jumpcommand = self.jumpdown
        else:
            self.jumpcommand = self.jumppress > 0 and not self.inair
            if self.jumpcommand:
                self.jumptimer = 0.295
        if self.attack is not None:
            coverage = 0.25 + 0.24*self.steam
            zone = 3.4 + 3.0*self.steam
            power = math.exp(self.steam)
            if self.attack == "thrust":
                a0,a1 = self.hwedge(coverage)
                h,k = self.scene.attack(0.1,zone,a0,a1,10*power,"thrust")
            elif self.attack == "slash":
                a0,a1 = self.hwedge(coverage)
                h,k = self.scene.attack(0.1,zone,a0,a1,15*power,"slash")
            elif self.attack == "upthrust":
                a0,a1 = self.upwedge(coverage)
                h,k = self.scene.attack(0.1,zone,a0,a1,10*power,"upthrust")
            elif self.attack == "upslash":
                a0,a1 = self.upwedge(coverage)
                h,k = self.scene.attack(0.1,zone,a0,a1,15*power,"upslash")
            elif self.attack == "flyingslash":
                a0,a1 = self.hwedge(coverage)
                h,k = self.scene.attack(0.1,zone,a0,a1,20*power,"flyingslash")
            elif self.attack == "flyingupslash":
                a0,a1 = self.upwedge(coverage)
                h,k = self.scene.attack(
                    0.1,zone,a0,a1,20*power,"flyinupgslash")
            elif self.attack == "flyingdownslash":
                a0,a1 = self.downwedge(coverage)
                h,k = self.scene.attack(
                    0.1,zone,a0,a1,20*power,"flyingdownslash")
            elif self.attack == "flyingthrust":
                a0,a1 = self.hwedge(coverage)
                h,k = self.scene.attack(
                    0.1,zone,a0,a1,12*power,"flyingthrust")
            elif self.attack == "flyingupthrust":
                a0,a1 = self.upwedge(coverage)
                h,k = self.scene.attack(
                    0.1,zone,a0,a1,12*power,"flyinupgthrust")
            elif self.attack == "flyingdownthrust":
                a0,a1 = self.downwedge(coverage)
                h,k = self.scene.attack(
                    0.1,zone,a0,a1,12*power,"flyingdownthrust")
            else:
                h = 0
                k = 0
            self.steam = max(self.steam-h*self.RELIEFRATE/self.tension,0.0)
            self.steamtimer = None
            self.kills += k
        elif self.firepress > 0:
            if self.inair:
                if abs(self.xvel) > self.SPEED*0.3:
                    if self.downdown:
                        self.attack = "flyingdownslash"
                        self.attacktimer = 0.495
                    elif self.updown:
                        self.attack = "flyingupslash"
                        self.attacktimer = 0.295
                    elif self.rightdown or self.leftdown:
                        self.attack = "flyingslash"
                        self.attacktimer = 0.495
                    else:
                        self.attack = "flyingthrust"
                        self.attacktimer = 0.495
                else:
                    if self.downdown:
                        self.attack = "flyingdownthrust"
                        self.attacktimer = 0.495
                    elif self.updown:
                        self.attack = "flyingupthrust"
                        self.attacktimer = 0.295
                    else:
                        self.attack = "flyingthrust"
                        self.attacktimer = 0.495
            else:
                if abs(self.xvel) > self.SPEED*0.5:
                    if self.updown:
                        self.attack = "upslash"
                        self.attacktimer = 0.295
                    else:
                        self.attack = "slash"
                        self.attacktimer = 0.295
                else:
                    if self.updown:
                        self.attack = "upthrust"
                        self.attacktimer = 0.295
                    else:
                        self.attack = "thrust"
                        self.attacktimer = 0.295
            self.steamtimer = None
        elif self.downdown and not self.inair and self.xvelcommand == 0:
            if self.steamtimer is None:
                self.steamtimer = 1.0
        else:
            self.steamtimer = None
        if self.recoiltimer is None:
            self.scene.be_attacked(2.0,4.0)
        self.downcommand = self.downdown
        self.firepress = 0
        self.jumppress = 0

    def nextframe(self):
        runframe = (self.runframe + 1) % 4

    def update(self,dt):
        if self.life <= 0.0:
            return
        if self.jumptimer is not None:
            self.jumptimer -= dt
            if self.jumptimer <= 0:
                self.jumptimer = None
        if self.attacktimer is not None:
            self.attacktimer -= dt
            if self.attacktimer <= 0:
                self.attacktimer = None
                self.attack = None
        if self.steamtimer is not None:
            self.steamtimer -= dt
            if self.steamtimer <= 0:
                self.steam = min(self.steam+self.STEAMRATE*self.tension*dt,1.0)
            self.steamframetimer -= dt
            if self.steamframetimer <= 0:
                self.steamframe = (self.steamframe + 1) % 4
                self.frametimer = self.STEAMFRAMETIME
        else:
            self.runframe = 0
        super(Heroine,self).update(dt)

    def draw(self):
        if self.life <= 0.0:
            return
        glPushMatrix()
        glTranslate(self.x,self.y,0.0)
        if self.facing == 'right':
            x0 = -1.6
            x1 = 3.4
            xs = 0.4
        else:
            x0 = 1.6
            x1 = -3.4
            xs = -0.4
        if self.steamtimer is not None:
            texture = self.textures["%s-heroine-steaming.png" % self.facing]
        elif not self.attack:
            if self.inair:
                frame = "leftstep"
            elif abs(self.xvel) < self.SPEED*0.2:
                frame = "standing"
            else:
                frame = self.RUNFRAMES[self.runframe]
            texture = self.textures["%s-heroine-%s.png" % (self.facing,frame)]
        elif self.attack == "thrust":
            texture = self.textures["%s-heroine-thrust.png" % self.facing]
        elif self.attack == "slash":
            texture = self.textures["%s-heroine-slash.png" % self.facing]
        elif self.attack in ("upthrust","flyingupthrust"):
            texture = self.textures["%s-heroine-hack2.png" % self.facing]
        elif self.attack in ("upslash","flyingupslash"):
            if self.attacktimer > 0.2:
                frame = "hack1"
            elif self.attacktimer > 0.1:
                frame = "hack2"
            else:
                frame = "hack3"
            texture = self.textures["%s-heroine-%s.png" % (self.facing,frame)]
        elif self.attack in ("flyingslash","flyingthrust"):
            if self.facing == "right":
                angle = -75.0
            else:
                angle = 75.0
            if self.attacktimer > 0.4:
                angle *= 0.5
                frame = "leftstep"
            elif self.attacktimer > 0.3:
                if self.attack == "flyingslash":
                    frame = "hack1"
                else:
                    frame = "hack3"
            elif self.attacktimer > 0.2:
                if self.attack == "flyingslash":
                    frame = "hack2"
                else:
                    frame = "hack3"
            elif self.attacktimer > 0.1:
                frame = "hack3"
            else:
                angle *= 0.5
                frame = "leftstep"
            glTranslate(0.0,self.HEIGHT/2,0.0)
            glRotate(angle,0.0,0.0,1.0)
            glTranslate(0.0,-self.HEIGHT/2,0.0)
            texture = self.textures["%s-heroine-%s.png" % (self.facing,frame)]
        elif self.attack in ("flyingdownslash","flyingdownthrust"):
            if self.facing == "right":
                angles = (-90.0, -135.0, -180.0, -215.0, -270.0)
            else:
                angles = (90.0, 135.0, 180.0, 215.0, 270.0)
            if self.attacktimer > 0.4:
                angle = angles[0]
                frame = "leftstep"
            elif self.attacktimer > 0.3:
                angle = angles[1]
                if self.attack == "flyingslash":
                    frame = "hack1"
                else:
                    frame = "hack3"
            elif self.attacktimer > 0.2:
                angle = angles[2]
                if self.attack == "flyingslash":
                    frame = "hack2"
                else:
                    frame = "hack3"
            elif self.attacktimer > 0.1:
                angle = angles[3]
                frame = "hack3"
            else:
                angle = angles[4]
                frame = "leftstep"
            glTranslate(0.0,self.HEIGHT/2,0.0)
            glRotate(angle,0.0,0.0,1.0)
            glTranslate(0.0,-self.HEIGHT/2,0.0)
            texture = self.textures["%s-heroine-%s.png" % (self.facing,frame)]
        texture.bind()
        glBegin(GL_QUADS)
        if self.recoiltimer:
            glColor(0.5,0.5,1.0,1.0)
        else:
            glColor(1.0,1.0,1.0,1.0)
        glTexCoord(0.0,0.0)
        glVertex(x0,0.0)
        glTexCoord(0.0,1.0)
        glVertex(x0,8.0)
        glTexCoord(1.0,1.0)
        glVertex(x1,8.0)
        glTexCoord(1.0,0.0)
        glVertex(x1,0.0)
        glEnd()
        texture.unbind()
        if self.steamtimer is not None and self.steamtimer <= 0.0:
            stexture = 'steam%d.png'%(self.steamframe+1)
            self.textures[stexture].bind()
            glBegin(GL_QUADS)
            glColor(1.0,1.0,1.0,1.0)
            glTexCoord(0.0,0.0)
            glVertex(xs-1.0,5.0)
            glTexCoord(0.0,1.0)
            glVertex(xs-1.0,7.0)
            glTexCoord(1.0,1.0)
            glVertex(xs+1.0,7.0)
            glTexCoord(1.0,0.0)
            glVertex(xs+1.0,5.0)
            glEnd()
            self.textures[stexture].unbind()
        glPopMatrix()

    def draw_overlay(self):
        glLoadIdentity()
        gluOrtho2D(0.0,40.0,0.0,30.0)
        glColor(1.0,1.0,1.0)
        self.textures['life.png'].bind()
        glBegin(GL_QUADS)
        glTexCoord(0.0,0.0)
        glVertex(1.0,28.2)
        glTexCoord(1.0,0.0)
        glVertex(4.0,28.2)
        glTexCoord(1.0,1.0)
        glVertex(4.0,29.2)
        glTexCoord(0.0,1.0)
        glVertex(1.0,29.2)
        glEnd()        
        self.textures['life.png'].unbind()
        self.textures['steam.png'].bind()
        glBegin(GL_QUADS)
        glTexCoord(0.0,0.0)
        glVertex(1.0,27.2)
        glTexCoord(1.0,0.0)
        glVertex(4.0,27.2)
        glTexCoord(1.0,1.0)
        glVertex(4.0,28.2)
        glTexCoord(0.0,1.0)
        glVertex(1.0,28.2)
        glEnd()
        self.textures['steam.png'].unbind()
        glBegin(GL_QUADS)
        glColor(0.1,0.2,1.0)
        glVertex(5.0,29.0)
        glVertex(5.0+10.0*self.life,29.0)
        glVertex(5.0+10.0*self.life,28.5)
        glVertex(5.0,28.5)
        glColor(1.0,0.2,0.2)
        glVertex(5.0,27.5)
        glVertex(5.0+10.0*self.steam,27.5)
        glVertex(5.0+10.0*self.steam,28.0)
        glVertex(5.0,28.0)
        glEnd()        
        glColor(0.0,0.0,0.0)
        glBegin(GL_LINE_LOOP)
        glVertex(5.0,29.0)
        glVertex(15.0,29.0)
        glVertex(15.0,28.5)
        glVertex(5.0,28.5)
        glEnd()
        glBegin(GL_LINE_LOOP)
        glVertex(5.0,27.5)
        glVertex(15.0,27.5)
        glVertex(15.0,28.0)
        glVertex(5.0,28.0)
        glEnd()
        self.textures['numbers.png'].bind()
        glBegin(GL_QUADS)
        glColor(1.0,1.0,1.0,1.0)
        x = 30.0
        for c in str(self.kills):
            n = ord(c)-48
            if not 0 <= n <= 9:
                continue
            glTexCoord(0.1*n,0.0)
            glVertex(x,27.5)
            glTexCoord(0.1*(n+1),0.0)
            glVertex(x+1.6,27.5)
            glTexCoord(0.1*(n+1),1.0)
            glVertex(x+1.6,29.5)
            glTexCoord(0.1*n,1.0)
            glVertex(x,29.5)
            x += 1.6
        glEnd()        
        self.textures['numbers.png'].unbind()
