# humanity.py

from __future__ import division

import math

import texturing

class Character(object):
    FRAMETIME = 0.095
    RECOILTIME = 0.995
    
    def __init__(self):
        self.textures = {}
        for texname in self.TEXTURE_NAMES:
            self.textures[texname] = texturing.Texture(texname)

    def unload(self):
        for texture in self.textures.itervalues():
            texture.unload()
        self.__dict__.clear()

    def enter_scene(self,scene,x,y):
        self.scene = scene
        self.inair = True
        self.x = x
        self.y = y
        self.xvel = 0.0
        self.yvel = 0.0
        self.facing = 'right'
        self.runframe = 0
        self.frametimer = self.FRAMETIME
        self.recoiltimer = None
        self.xvelcommand = 0.0
        self.jumpcommand = False
        self.downcommand = False

    def exit_scene(self):
        del self.scene
        del self.inair
        del self.x
        del self.y
        del self.xvel
        del self.yvel
        del self.facing
        del self.runframe
        del self.frametimer
        del self.xvelcommand
        del self.jumpcommand
        del self.downcommand

    def recoil(self,xfrom,yfrom):
        self.recoiltimer = self.RECOILTIME
        if self.facing == "right":
            self.xvel -= 30.0
        else:
            self.xvel += 30.0
        self.yvel += 30.0

    def update(self,dt):
        if self.life <= 0.0:
            return
        if self.recoiltimer is not None:
            self.recoiltimer -= dt
            if self.recoiltimer <= 0:
                self.recoiltimer = None
        if self.frametimer is not None:
            self.frametimer -= dt
            if self.frametimer <= 0:
                self.runframe = (self.runframe + 1) % len(self.RUNFRAMES)
                self.frametimer = self.FRAMETIME
        else:
            self.runframe = 0
        xdefect = self.xvelcommand - self.xvel
        if self.inair:
            maxdxv = self.SPEED/0.3*dt
        else:
            maxdxv = self.SPEED/0.2*dt            
        if abs(xdefect) <= maxdxv:
            self.xvel = self.xvelcommand
        elif self.xvel < self.xvelcommand:
            self.xvel += maxdxv
        else:
            self.xvel -= maxdxv
        if self.inair:
            self.yvel -= self.GRAVITY*dt
            if self.jumpcommand:
                self.yvel += self.JUMP*dt
        elif self.jumpcommand:
            self.yvel += self.JUMP*dt
        elif self.recoiltimer is None:
            self.yvel = 0.0
        self.x,self.y,self.inair = self.scene.move(
            self.x,self.y,self.xvel*dt,self.yvel*dt,self.downcommand)
