# texturing.py

from __future__ import division

import os

from PIL import Image
from OpenGL.GL import *
from OpenGL.GLU import *


class TextureInternal(object):
    def __init__(self,filename):
        self.filename = filename
        image = Image.open(os.path.join('png',self.filename))
        image = image.transpose(Image.FLIP_TOP_BOTTOM)
        width,height = image.size
        self.ti = glGenTextures(1)
        glBindTexture(GL_TEXTURE_2D,self.ti)
        glTexParameter(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,
                       GL_LINEAR_MIPMAP_LINEAR)
        glTexParameter(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR)
        glTexParameter(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT)
        glTexParameter(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT)
        gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGBA,width,height,GL_RGBA,
                          GL_UNSIGNED_BYTE,image.tostring())
    def unload(self):
        glDeleteTextures((self.ti,))
    def bind(self):
        glBindTexture(GL_TEXTURE_2D,self.ti)
        glEnable(GL_TEXTURE_2D)
    def unbind(self):
        glDisable(GL_TEXTURE_2D)


class Texture(object):
    cache = {}
    refcount = {}    
    def __init__(self,filename):
        if filename not in self.cache:
            self.cache[filename] = TextureInternal(filename)
            self.refcount[filename] = 0
        self.refcount[filename] += 1
        self.texture = self.cache[filename]
    def unload(self):
        filename = self.texture.filename
        self.refcount[filename] -= 1
        if self.refcount[filename] <= 0:
            del self.cache[filename]
            del self.refcount[filename]
            self.texture.unload()
    def bind(self):
        self.texture.bind()
    def unbind(self):
        self.texture.unbind()
